@@ -4,7 +4,7 @@
  *	  prototypes for pquery.c.
  *
  *
- * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/tcop/pquery.h
@@ -28,18 +28,18 @@ extern List *FetchPortalTargetList(Portal portal);
 extern List *FetchStatementTargetList(Node *stmt);
 
 extern void PortalStart(Portal portal, ParamListInfo params,
-			int eflags, Snapshot snapshot);
+						int eflags, Snapshot snapshot);
 
 extern void PortalSetResultFormat(Portal portal, int nFormats,
-					  int16 *formats);
+								  int16 *formats);
 
 extern bool PortalRun(Portal portal, long count, bool isTopLevel,
-		  bool run_once, DestReceiver *dest, DestReceiver *altdest,
-		  char *completionTag);
+					  bool run_once, DestReceiver *dest, DestReceiver *altdest,
+					  QueryCompletion *qc);
 
 extern uint64 PortalRunFetch(Portal portal,
-			   FetchDirection fdirection,
-			   long count,
-			   DestReceiver *dest);
+							 FetchDirection fdirection,
+							 long count,
+							 DestReceiver *dest);
 
 #endif							/* PQUERY_H */
